package BLEU;

# ------------------------------------------------------------------------

#Copyright (C) 2005 Enrique Amig and Jess Gimnez

#This library is free software; you can redistribute it and/or
#modify it under the terms of the GNU Lesser General Public
#License as published by the Free Software Foundation; either
#version 2.1 of the License, or (at your option) any later version.

#This library is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#Lesser General Public License for more details.

#You should have received a copy of the GNU Lesser General Public
#License along with this library; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

# ------------------------------------------------------------------------

use strict;
use Data::Dumper;
use IQ::IQXML;
use IQ::COMMON;
use File::ReadBackwards;

use vars qw($BLEUEXT $rBLEU $TBLEU);

$BLEU::BLEUEXT = "BLEU";
$BLEU::TBLEU = "mteval-kit/mteval-v11b.pl -b -d 2"; # version v10
$BLEU::rBLEU = { "$BLEU::BLEUEXT-1" => 1, "$BLEU::BLEUEXT-2" => 1, "$BLEU::BLEUEXT-3" => 1, "$BLEU::BLEUEXT-4" => 1};

sub read_bleu {
   #description _ read BLEU value from report file
   #param1  _ report filename
   #@return _ BLEU score list (n = 1..9)

   my $report = shift;

   my @lbleu;
   my $REPORT = File::ReadBackwards->new($report) or die "Couldn't open input file: $report\n";
   my $line = $REPORT->readline;
   while (!($line =~ /^ +BLEU:.*/)) { $line = $REPORT->readline; }
   $REPORT->close();
   chomp($line);
   $line =~ s/^ +BLEU: +//; $line =~ s/ +/ /g;
   my @lbleu = split(" ", $line);
   pop(@lbleu);

   return \@lbleu;
}

sub read_bleu_segments {
   #description _ read BLEU-4 value from report file (for all segments)
   #param1  _ report filename
   #@return _ bleu score list

   my $report = shift;

   my @lbleu1;
   my @lbleu2;
   my @lbleu3;
   my @lbleu4;

   open(AUX, "< $report") or die "couldn't open file: $report\n";

   while (my $aux = <AUX>) {
      if ($aux =~ /^ +BLEU score using.*/) {
         chomp($aux);
         my @laux = split(/ +/, $aux);
         my @lN;
         push(@lbleu1, $laux[6]);
         push(@lbleu2, $laux[7]);
         push(@lbleu3, $laux[8]);
         push(@lbleu4, $laux[9]);
      }
   }
   close(AUX);
   my @SEG = (\@lbleu1, \@lbleu2, \@lbleu3, \@lbleu4);

   return \@SEG;
}

sub computeMultiBLEU {
   #description _ computes BLEU score (by calling NIST mteval script) -> n = 1..4 (multiple references)
   #param1 _ source file
   #param2 _ candidate file
   #param3 _ reference file(s) [hash reference]
   #param4 _ remake reports? (1 - yes :: 0 - no)
   #param5 _ TARGET NAME (base filename)
   #param6 _ REF NAME (base filename)
   #param7 _ tools
   #param8 _ verbosity (0/1)

   my $src = shift;
   my $out = shift;
   my $Href = shift;
   my $remakeREPORTS = shift;
   my $TGTname = shift;
   my $REFname = shift;
   my $tools = shift;
   my $verbose = shift;

   my $toolBLEU = "perl $tools/$BLEU::TBLEU";

   my $refBLEUsgml = "./$COMMON::TMP/".rand($COMMON::NRAND).".$COMMON::REFEXT.$BLEUEXT.$COMMON::SGMLEXT";
   if ((!(-e $refBLEUsgml)) or $remakeREPORTS) { IQXML::NIST_f_create_mteval_multidoc($Href, $refBLEUsgml, 2, $verbose); }
   my $srcBLEUsgml = "./$COMMON::TMP/".rand($COMMON::NRAND).".$COMMON::SRCEXT.$BLEUEXT.$COMMON::SGMLEXT";
   if ((!(-e $srcBLEUsgml)) or $remakeREPORTS) { IQXML::NIST_f_create_mteval_doc($src, $srcBLEUsgml, 0, $verbose); }
   my $outBLEUsgml = "./$COMMON::TMP/".rand($COMMON::NRAND).".$COMMON::SYSEXT.$BLEUEXT.$COMMON::SGMLEXT";
   if ((!(-e $outBLEUsgml)) or $remakeREPORTS) { IQXML::NIST_f_create_mteval_doc($out, $outBLEUsgml, 1, $verbose); }
   my $reportBLEU = "./$COMMON::TMP/".rand($COMMON::NRAND).".$BLEUEXT.$COMMON::REPORTEXT";
   if ($verbose > 1) { print STDERR "building $reportBLEU...\n"; }
   system("$toolBLEU -s $srcBLEUsgml -t $outBLEUsgml -r $refBLEUsgml > $reportBLEU");
   if (-e $srcBLEUsgml) { system "rm -f $srcBLEUsgml"; }
   if (-e $refBLEUsgml) { system "rm -f $refBLEUsgml"; }
   if (-e $outBLEUsgml) { system "rm -f $outBLEUsgml"; }
   my $SYS = read_bleu($reportBLEU);
   my $SEG = read_bleu_segments($reportBLEU);
   system("rm -f $reportBLEU");

   return($SYS, $SEG);
}

sub doMultiBLEU {
   #description _ computes BLEU score (by calling NIST mteval script) -> n = 1..4 (multiple references)
   #param1  _ source file
   #param2  _ candidate file
   #param3  _ reference file(s) [hash reference]
   #param4  _ remake reports? (1 - yes :: 0 - no)
   #param5  _ TARGET NAME (base filename)
   #param6  _ REF NAME (base filename)
   #param7  _ tools
   #param8  _ metrics to compute
   #param9  _ optional prefix
   #param10 _ verbosity (0/1)

   my $src = shift;
   my $out = shift;
   my $Href = shift;
   my $remakeREPORTS = shift;
   my $TGTname = shift;
   my $REFname = shift;
   my $tools = shift;
   my $M = shift;
   my $prefix = shift;
   my $verbose = shift;

   my $GO = 0; my $i = 0;
   my @mBLEU = keys %{$BLEU::rBLEU};
   while (($i < scalar(@mBLEU)) and (!$GO)) { if (exists($M->{$prefix.$mBLEU[$i]})) { $GO = 1; } $i++; }

   if ($GO) {
      if ($verbose == 1) { print STDERR "BLEU.."; }
      my $reportBLEU1xml = "./$TGTname/$REFname/$prefix$BLEUEXT-1.$COMMON::XMLEXT";
      my $reportBLEU2xml = "./$TGTname/$REFname/$prefix$BLEUEXT-2.$COMMON::XMLEXT";
      my $reportBLEU3xml = "./$TGTname/$REFname/$prefix$BLEUEXT-3.$COMMON::XMLEXT";
      my $reportBLEU4xml = "./$TGTname/$REFname/$prefix$BLEUEXT-4.$COMMON::XMLEXT";

      if ((!(-e $reportBLEU1xml) and !(-e $reportBLEU1xml.".$COMMON::GZEXT")) or (!(-e $reportBLEU2xml) and !(-e $reportBLEU2xml.".$COMMON::GZEXT")) or (!(-e $reportBLEU3xml) and !(-e $reportBLEU3xml.".$COMMON::GZEXT")) or (!(-e $reportBLEU4xml) and !(-e $reportBLEU4xml.".$COMMON::GZEXT")) or $remakeREPORTS) {
         my ($SYS, $SEG) = BLEU::computeMultiBLEU($src, $out, $Href, $remakeREPORTS, $TGTname, $REFname, $tools, $verbose);
         IQXML::write_report($TGTname, $REFname, "$prefix$BLEUEXT-1", $SYS->[0], $SEG->[0], $reportBLEU1xml, $verbose);
         IQXML::write_report($TGTname, $REFname, "$prefix$BLEUEXT-2", $SYS->[1], $SEG->[1], $reportBLEU2xml, $verbose);
         IQXML::write_report($TGTname, $REFname, "$prefix$BLEUEXT-3", $SYS->[2], $SEG->[2], $reportBLEU3xml, $verbose);
         IQXML::write_report($TGTname, $REFname, "$prefix$BLEUEXT-4", $SYS->[3], $SEG->[3], $reportBLEU4xml, $verbose);
      }
   }
}

sub computeBLEU {
   #description _ computes BLEU score (by calling NIST mteval script) -> n = 1..4 (multiple references)
   #param1 _ source file
   #param2 _ candidate file
   #param3 _ reference file(s) [hash reference]
   #param4 _ remake reports? (1 - yes :: 0 - no)
   #param5 _ TARGET NAME (base filename)
   #param6 _ REF NAME (base filename)
   #param7 _ tools
   #param8 _ verbosity (0/1)

   my $src = shift;
   my $out = shift;
   my $ref = shift;
   my $remakeREPORTS = shift;
   my $TGTname = shift;
   my $REFname = shift;
   my $tools = shift;
   my $verbose = shift;

   my $toolBLEU = "perl $tools/$BLEU::TBLEU";

   my $refBLEUsgml = "./$COMMON::TMP/".rand($COMMON::NRAND).".$COMMON::REFEXT.$BLEUEXT.$COMMON::SGMLEXT";
   if ((!(-e $refBLEUsgml)) or $remakeREPORTS) { IQXML::NIST_f_create_mteval_doc($ref, $refBLEUsgml, 2, $verbose); }
   my $srcBLEUsgml = "./$COMMON::TMP/".rand($COMMON::NRAND).".$COMMON::SRCEXT.$BLEUEXT.$COMMON::SGMLEXT";
   if ((!(-e $srcBLEUsgml)) or $remakeREPORTS) { IQXML::NIST_f_create_mteval_doc($src, $srcBLEUsgml, 0, $verbose); }
   my $outBLEUsgml = "./$COMMON::TMP/".rand($COMMON::NRAND).".$COMMON::SYSEXT.$BLEUEXT.$COMMON::SGMLEXT";
   if ((!(-e $outBLEUsgml)) or $remakeREPORTS) { IQXML::NIST_f_create_mteval_doc($out, $outBLEUsgml, 1, $verbose); }
   my $reportBLEU = "./$COMMON::TMP/".rand($COMMON::NRAND).".$BLEUEXT.$COMMON::REPORTEXT";
   if ($verbose > 1) { print STDERR "building $reportBLEU...\n"; }
   system("$toolBLEU -s $srcBLEUsgml -t $outBLEUsgml -r $refBLEUsgml > $reportBLEU");
   if (-e $srcBLEUsgml) { system "rm -f $srcBLEUsgml"; }
   if (-e $refBLEUsgml) { system "rm -f $refBLEUsgml"; }
   if (-e $outBLEUsgml) { system "rm -f $outBLEUsgml"; }
   my $SYS = read_bleu($reportBLEU);
   my $SEG = read_bleu_segments($reportBLEU);
   system("rm -f $reportBLEU");

   return($SYS, $SEG);
}

sub doBLEU {
   #description _ computes BLEU score (by calling NIST mteval script) -> n = 1..4 (single reference)
   #param1  _ source file
   #param2  _ candidate file
   #param3  _ reference file 
   #param4  _ remake reports? (1 - yes :: 0 - no)
   #param5  _ target NAME (base filename)
   #param6  _ reference NAME (base filename)
   #param7  _ tools
   #param8  _ metrics to compute
   #param9  _ optional prefix
   #param10 _ verbosity (0/1)

   my $src = shift;
   my $out = shift;
   my $ref = shift;
   my $remakeREPORTS = shift;
   my $TGTname = shift;
   my $REFname = shift;
   my $tools = shift;
   my $M = shift;
   my $prefix = shift;
   my $verbose = shift;

   my $GO = 0; my $i = 0;
   my @mBLEU = keys %{$BLEU::rBLEU};
   while (($i < scalar(@mBLEU)) and (!$GO)) { if (exists($M->{$prefix.$mBLEU[$i]})) { $GO = 1; } $i++; }

   if ($GO) {
      if ($verbose == 1) { print STDERR "BLEU.."; }
      my $reportBLEU1xml = "./$TGTname/$REFname/$prefix$BLEUEXT-1.$COMMON::XMLEXT";
      my $reportBLEU2xml = "./$TGTname/$REFname/$prefix$BLEUEXT-2.$COMMON::XMLEXT";
      my $reportBLEU3xml = "./$TGTname/$REFname/$prefix$BLEUEXT-3.$COMMON::XMLEXT";
      my $reportBLEU4xml = "./$TGTname/$REFname/$prefix$BLEUEXT-4.$COMMON::XMLEXT";

      if ((!(-e $reportBLEU1xml) and !(-e $reportBLEU1xml.".$COMMON::GZEXT")) or (!(-e $reportBLEU2xml) and !(-e $reportBLEU2xml.".$COMMON::GZEXT")) or (!(-e $reportBLEU3xml) and !(-e $reportBLEU3xml.".$COMMON::GZEXT")) or (!(-e $reportBLEU4xml) and !(-e $reportBLEU4xml.".$COMMON::GZEXT")) or $remakeREPORTS) {
         my ($SYS, $SEG) = BLEU::computeBLEU($src, $out, $ref, $remakeREPORTS, $TGTname, $REFname, $tools, $verbose);
         IQXML::write_report($TGTname, $REFname, "$prefix$BLEUEXT-1", $SYS->[0], $SEG->[0], $reportBLEU1xml, $verbose);
         IQXML::write_report($TGTname, $REFname, "$prefix$BLEUEXT-2", $SYS->[1], $SEG->[1], $reportBLEU2xml, $verbose);
         IQXML::write_report($TGTname, $REFname, "$prefix$BLEUEXT-3", $SYS->[2], $SEG->[2], $reportBLEU3xml, $verbose);
         IQXML::write_report($TGTname, $REFname, "$prefix$BLEUEXT-4", $SYS->[3], $SEG->[3], $reportBLEU4xml, $verbose);
      }
   }
}

1;
